function ReadRawData(fid, precision);

% readRawData reads an EGI epoch-marked raw format file.
%
% Epoch-marked raw format: Unsegmented simple binary format, version # 2, 4 or 6.
%
% The single sample records (SSR) are extracted to form the x matrix,
% with one SSR per column.
%
% The corresponding event records, one per SSR, are extracted to form the
% eventData matrix, with one event record per column.
%
% Input Arguments: 
%
%   fid         - EEG data file ID from fopen.
%
%   precision   - Input numeric precision string. 
% 
% Output Arguments:
%
%   x           - (Global Variable) Array of EEG data, channels x samples.
%
%   events      - (Global Variable) Array of event code flags, codes x samples.
%
%   metaData    - (Global Variable) Structure array containing the files meta data.

global metaData x events;

switch precision
    case 'integer*2'
        numBytes = 2;
    case 'real*4'
        numBytes = 4;
    case 'real*8'
        numBytes = 8;
end

% Read SSRs into array x, corresponding events into array eventData. ------------------------------

numRead1 = 0;
x = zeros(metaData.numCh, metaData.numSamples);

if metaData.numEvents

    errFlag   =  0;
    numRead2  =  0;
    events    =  zeros(metaData.numEvents, metaData.numSamples);

    for j = 1 : metaData.numSamples

        [x(:,j), tempCntr1] = fread(fid, metaData.numCh, precision);
        numRead1 = numRead1 + tempCntr1;

        [events(:,j), tempCntr2] = fread(fid, metaData.numEvents, precision);
        numRead2 = numRead2 + tempCntr2;

    end

    if (numRead1 ~= metaData.numCh * metaData.numSamples)

        errMsg1 = 'Error (File Read): EEG Data Array';
        errFlag = 1;

    else
        
        errMsg1 = [];
        
    end
    
    if (numRead2 ~= metaData.numEvents * metaData.numSamples)
      
        errMsg2 = 'Error (File Read): Event Data Array';
        errFlag = 1;
       
    else
        
        errMsg2 = [];
        
    end
    
    if errFlag
        
        errordlg( strvcat(errMsg1, errMsg2) , 'File Read Error' );
        return;
        
    end
    
    for i = 1 : metaData.numEvents
        
        epoc = strcmpi('epoc', char(metaData.eventCodes{i}));
        
        if epoc
            
            metaData.epocIX = i;
            metaData.numEpoc = sum(events(metaData.epocIX,:));
            break;
            
        end
        
    end
    
else
        
    [x, numRead1] = fread(fid, [metaData.numCh, metaData.numSamples], precision);

    if (numRead1 ~= metaData.numCh * metaData.numSamples)

        errordlg( 'Error (File Read): EEG Data Array' , 'File Read Error' );
        return;

    end

end

% Finished reading SSRs. --------------------------------------------------------------------------

fclose(fid);
